@@ -1493,9 +1493,24 @@ BEGIN_TEST(insert_nops.setpc_gfx11)
    /* VALUMaskWriteHazard */
    //! p_unit_test 4
    //! v1: %0:v[0] = v_cndmask_b32 0, 0, %0:vcc
+   //! s1: %0:vcc_hi = s_mov_b32 0
    //! s_waitcnt_depctr va_vdst(0) sa_sdst(0)
+   //! v1: %0:v[0] = v_xor3_b32 %0:v[0], %0:s[0], %0:s[0]
+   //! s_waitcnt_depctr va_vdst(0)
    //! s_setpc_b64 0
    bld.pseudo(aco_opcode::p_unit_test, Operand::c32(4));
+   bld.vop2(aco_opcode::v_cndmask_b32, Definition(PhysReg(256), v1), Operand::zero(),
+            Operand::zero(), Operand(vcc, s2));
+   bld.sop1(aco_opcode::s_mov_b32, Definition(vcc_hi, s1), Operand::c32(0));
+   bld.sop1(aco_opcode::s_setpc_b64, Operand::zero(8));
+
+   //! p_unit_test 8
+   //! v1: %0:v[0] = v_cndmask_b32 0, 0, %0:vcc
+   //! s_waitcnt_depctr va_vdst(0)
+   //! v1: %0:v[0] = v_xor3_b32 %0:v[0], %0:s[0], %0:s[0]
+   //! s_waitcnt_depctr va_vdst(0)
+   //! s_setpc_b64 0
+   bld.pseudo(aco_opcode::p_unit_test, Operand::c32(8));
    bld.vop2(aco_opcode::v_cndmask_b32, Definition(PhysReg(256), v1), Operand::zero(),
             Operand::zero(), Operand(vcc, s2));
    bld.sop1(aco_opcode::s_setpc_b64, Operand::zero(8));
@@ -1504,6 +1519,8 @@ BEGIN_TEST(insert_nops.setpc_gfx11)
    //! v1: %0:v[0] = v_cndmask_b32 0, 0, %0:vcc
    //! s2: %0:vcc = s_mov_b64 0
    //! s_waitcnt_depctr va_vdst(0) sa_sdst(0)
+   //! v1: %0:v[0] = v_xor3_b32 %0:v[0], %0:s[0], %0:s[0]
+   //! s_waitcnt_depctr va_vdst(0)
    //! s_setpc_b64 0
    bld.pseudo(aco_opcode::p_unit_test, Operand::c32(5));
    bld.vop2(aco_opcode::v_cndmask_b32, Definition(PhysReg(256), v1), Operand::zero(),
