@@ -19,7 +19,11 @@ extern "C" {
 % for name, enum in enums.items():
 enum PACKED ${prefix}_${name} {
 % for k, v in enum.items():
+   % if v:
    ${k} = ${v},
+   % else:
+   ${k},
+   %endif
 % endfor
 };
 
@@ -30,6 +34,7 @@ enum PACKED ${prefix}_${name} {
 #endif
 """
 
+import argparse
 import sys, os
 sys.path.append(os.path.dirname(os.path.abspath(__file__)) + "/../../compiler/isaspec")
 
@@ -37,7 +42,14 @@ from mako.template import Template
 from isa import ISA
 
 def main():
-    isa = ISA(sys.argv[1])
+    parser = argparse.ArgumentParser()
+    parser.add_argument('--xml', required=True, type=str, action="store",
+                        help='source isaspec xml file')
+    parser.add_argument('--output', required=True, type=str, action="store",
+                        help='output C header file')
+    args = parser.parse_args()
+
+    isa = ISA(args.xml)
     prefix = 'isa'
     enums = {}
 
@@ -55,17 +67,12 @@ def main():
     opc = {}
 
     for instr in isa.instructions():
-        pattern = instr.xml.findall('pattern')
-        bit05 = int(pattern[0].text, 2)
-        bit6 = int(pattern[1].text, 2)
-        num = bit05 | (bit6 << 6)
+        opc[prefix.upper() + '_OPC_' + instr.name.upper()] = None
 
-        opc[prefix.upper() + '_OPC_' + instr.name.upper()] = num
-
-    opc = dict(sorted(opc.items(), key=lambda item: int(item[1])))
     enums['opc'] = opc
 
-    print(Template(template).render(prefix=prefix, enums=enums))
+    with open(args.output, "w", encoding="UTF-8") as fh:
+        fh.write(Template(template).render(prefix=prefix, enums=enums))
 
 if __name__ == '__main__':
     main()
